#-*- coding: utf-8 -*-
import html
import json
import re
import os
from urllib import parse
from Doraemon.Requests import requests_dora
from tqdm import tqdm

def get_album_list(sin, ein):
    url = "https://c.y.qq.com/splcloud/fcgi-bin/fcg_get_diss_by_tag.fcg"
    header = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,  like Gecko) Chrome/59.0.3071.115 Safari/537.36",
        "cookie": "RK=7dNm4/X + Yj; tvfe_boss_uuid=bf00ee54e9081ab4; pgv_pvi=8772238336; pac_uid=1_857193777; pgv_pvid=6457341280; o_cookie=80; ptcz=c761e59c8c8d6bd5198866d02a5cb7313af1af468006c455d6c2b5d26201d42e; pgv_si=s10759168; _qpsvr_localtk=0.08285763449905015; ptisp=ctc; luin=o0857193777; lskey=00010000228dd1371b945c68ecfd3b71d3071425024a7a8a2a23e3ffcb5b9904c9f7088d2ea8c01539ffed92; pt2gguin=o0857193777; uin=o0857193777; skey=@Kydi7w0EI; p_uin=o0857193777; p_skey=HjsE9sEjznJfXk*9KFEeW4VZr6i3*tlXZ2nuzEw8kCg_; pt4_token=c-p6sv3JEboA51cSQ3ABqxM8O80Jct3jYYkgy-aEQuE_; p_luin=o0857193777; p_lskey=000400008f9c296cd10c03a5173d22a184aad124d791568e90e4198beb8ad699a4d02fbfc059f71ab3d8758c; ts_last=y.qq.com/portal/playlist.html; ts_refer=ui.ptlogin2.qq.com/cgi-bin/login; ts_uid=3392060960",
        "referer": "https://y.qq.com/portal/playlist.html"
    }
    paramter = {
        "g_tk": "1089387893",
        "jsonpCallback": "getPlaylist",
        "loginUin": "0",
        "hostUin": "0",
        "format": "jsonp",
        "inCharset": "utf8",
        "outCharset": "utf-8",
        "notice": "0",
        "platform": "yqq",
        "needNewCode": "0",
        "categoryId": "10000000",
        "sortId": "5",
        "sin": sin, # 开始结点
        "ein": ein # 结束结点，用于翻页
    }
    html_text = requests_dora.try_best_2_get(url=url, params=paramter, headers=header).text
    res = json.loads(html_text.lstrip("getPlaylist(").rstrip(")"))["data"]["list"]
    album_list = []

    for t_item in res:
        album = {}
        ILLEGAL_CHARACTERS_RE = re.compile(r"[\000-\010]|[\013-\014]|[\016-\037]")#用于去掉非法字符
        album["createtime"] = t_item["createtime"]
        album["creator_qq"] = t_item["creator"]["qq"]
        album["creator_name"] = t_item["creator"]["name"]
        album["creator_name"] = ILLEGAL_CHARACTERS_RE.sub(r"",  album["creator_name"])
        album["creator_isVip"] = t_item["creator"]["isVip"]
        album["dissid"] = t_item["dissid"] #提取歌单id，用于后续提取歌曲id
        album["dissname"] = t_item["dissname"] #歌单名称
        album["dissname"] = ILLEGAL_CHARACTERS_RE.sub(r"",  album["dissname"])
        album["listennum"] = t_item["listennum"] #播放量
        album_list.append(album)
    return album_list


#爬取歌曲id
def get_song_list(dissid):
    url = "https://c.y.qq.com/qzone/fcg-bin/fcg_ucc_getcdinfo_byids_cp.fcg"
    header = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,  like Gecko) Chrome/59.0.3071.115 Safari/537.36",
        "cookie": "RK=7dNm4/X + Yj; tvfe_boss_uuid=bf00ee54e9081ab4; pgv_pvi=8772238336; pac_uid=1_857193777; pgv_pvid=6457341280; o_cookie=857193777; ptcz=c761e59c8c8d6bd5198866d02a5cb7313af1af468006c455d6c2b5d26201d42e; pgv_si=s10759168; _qpsvr_localtk=0.08285763449905015; ptisp=ctc; luin=o0857193777; lskey=00010000228dd1371b945c68ecfd3b71d3071425024a7a8a2a23e3ffcb5b9904c9f7088d2ea8c01539ffed92; pt2gguin=o0857193777; uin=o0857193777; skey=@Kydi7w0EI; p_uin=o0857193777; p_skey=HjsE9sEjznJfXk*9KFEeW4VZr6i3*tlXZ2nuzEw8kCg_; pt4_token=c-p6sv3JEboA51cSQ3ABqxM8O80Jct3jYYkgy-aEQuE_; p_luin=o0857193777; p_lskey=000400008f9c296cd10c03a5173d22a184aad124d791568e90e4198beb8ad699a4d02fbfc059f71ab3d8758c; ts_last=y.qq.com/portal/playlist.html; ts_refer=ui.ptlogin2.qq.com/cgi-bin/login; ts_uid=3392060960",
        "referer": "https://y.qq.com/n/yqq/playlist/{}.html".format(dissid)
    }
    paramters = {
        "type": "1",
        "json": "1",
        "utf8": "1",
        "onlysong": "0",
        "disstid": dissid,
        "format": "jsonp",
        "g_tk": "1089387893",
        "jsonpCallback": "playlistinfoCallback",
        "loginUin": "857193777",
        "hostUin": "0",
        "inCharset": "utf8",
        "outCharset": "utf-8",
        "notice": 0,
        "platform": "yqq",
        "needNewCode": 0
    }
    html_text = requests_dora.try_best_2_get(url=url, params=paramters, headers=header).text
    cdlist = json.loads(html_text.lstrip("playlistinfoCallback(").rstrip(")"))["cdlist"]
    if len(cdlist) >= 1:
        cdlist = cdlist[0]
    song_list = []

    tags = ", ".join([i["name"] for i in cdlist["tags"]])
    for item in cdlist["songlist"]:
        song = {}
        # if "size128" in item:
        #     song["size128"] = item["size128"]
        if "songmid" in item:
            song["songmid"] = item["songmid"]
        else:
            continue

        if "songid" in item:
            song["songid"] = item["songid"]
        else:
            continue

        song["albumname"] = item["albumname"]
        song["songname"] = item["songname"]
        song["singer"] = ", ".join([i["name"] for i in item["singer"]])
        song["tags"] = tags
        song_list.append(song)
    return song_list


def get_lyric(song):
    songid = song["songid"]
    songmid = song["songmid"]
    url = "https://c.y.qq.com/lyric/fcgi-bin/fcg_query_lyric.fcg"
    header = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,  like Gecko) Chrome/59.0.3071.115 Safari/537.36",
        "referer": "https://y.qq.com/n/yqq/song/{}.html".format(songmid)
    }
    paramters = {
        "nobase64": 1,
        "musicid": songid,
        "callback": "jsonp1",
        "g_tk": "1134533366",
        "jsonpCallback": "jsonp1", 
        "loginUin": "0",
        "hostUin": "0",
        "format": "jsonp",
        "inCharset": "utf8",
        "outCharset": "utf-8",
        "notice": "0",
        "platform": "yqq",
        "needNewCode": "0"
    }
    html_text = requests_dora.try_best_2_get(url=url, params=paramters, headers=header).text
    res = json.loads(html_text.lstrip("jsonp1(").rstrip(")"))
    # if "lyric" in res:
    #     lyric = res["lyric"]
    #     # decode
    #     lyric = html.unescape(lyric)
    #     lyric = html.unescape(lyric)
    #     lyric = parse.unquote(lyric)
    #
    #     it = re.finditer(r"\[(.*?)\](.+)", lyric)
    #     lyric_lines = []
    #     for match in it:
    #         time_pop_up = match.group(1)
    #         time_split = time_pop_up.split(".")
    #         ms = float("0.{}".format(time_split[1]))
    #         sec = time.strptime(time_split[0], "%M:%S").tm_sec
    #         line = match.group(2)
    #         line = line.strip()
    #         if re.search("[:：]", line) or line == "" or line == "此歌曲为没有填词的纯音乐，请您欣赏":
    #             continue
    #         lyric_lines.append({
    #             "time": sec + ms,
    #             "line": line,
    #         })
    #
    #     return lyric_lines[1:]
    # else:
    #     return []
    if "lyric" in res:
        lyric = res["lyric"]
        if "此歌曲为没有填词的纯音乐，请您欣赏" in lyric:
            return {}, []
        # decode
        lyric = html.unescape(lyric)
        lyric = html.unescape(lyric)
        lyric = parse.unquote(lyric)

        it = re.finditer(r"\[(\d+):(\d+.\d+)\](.+)", lyric)
        lyric_lines = []
        contributors_dict = {}
        for match in it:
            min = float(match.group(1))
            try:
                sec = float(match.group(2))
            except ValueError:
                sec = 0

            line = match.group(3)
            line = line.strip()
            if line == "":
                continue
            se_contributors = re.search("(.*?)[:：](.*)", line)
            if se_contributors:
                contributors_dict[se_contributors.group(1).strip()] = se_contributors.group(2).strip()
                continue
            lyric_lines.append({
                "time": min * 60 + sec,
                "line": line,
            })

        return contributors_dict, lyric_lines[1:]
    else:
        return {}, []


def get_detail(song):
    songid = song["songid"]
    songmid = song["songmid"]

    url = "https://c.y.qq.com/v8/fcg-bin/fcg_play_single_song.fcg"
    header = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,  like Gecko) Chrome/59.0.3071.115 Safari/537.36",
        "referer": "https://y.qq.com/n/yqq/song/{}.html".format(songid)
    }
    paramters = {
        "songmid": songmid,
        "tpl": "yqq_song_detail",
        "format": "jsonp",
        "callback": "getOneSongInfoCallback",
        "g_tk": "1134533366",
        "jsonpCallback": "getOneSongInfoCallback",
        "loginUin": "0",
        "hostUin": "0",
        "inCharset": "utf8",
        "outCharset": "utf-8",
        "notice": 0,
        "platform": "yqq",
        "needNewCode": 0
    }
    html_text = requests_dora.try_best_2_get(url=url, params=paramters, headers=header, verify=True).text
    detail = json.loads(html_text.lstrip("getOneSongInfoCallback(").rstrip(")"))["data"]
    song = {}
    if len(detail) > 0:
        detail = detail[0]
        song["subtitle"] = detail["subtitle"]
        song["title"] = detail["title"]
        song["time_public"] = detail["time_public"]
        try:
            song["url"] = json.loads(html_text.lstrip("getOneSongInfoCallback(").rstrip(")"))["url"][str(songid)]
        except:
            song["url"] = ""
    return song


if __name__ == "__main__":
    save_path = "./qq_music_songs_by_album"
    if not os.path.exists(save_path):
        os.makedirs(save_path)

    for sin in range(0, 7050, 30):
        ein = sin + 29
        album_list = get_album_list(sin, ein) # get 30 albums

        for album in album_list:
            dissname = album["dissname"]
            song_list = get_song_list(album["dissid"])
            chunk = []
            for song in tqdm(song_list, desc = "getting songs in {}".format(dissname)):
                contributors, lyric = get_lyric(song)
                song["lyric"] = lyric
                chunk.append(song)

            json.dump(chunk, open("{}/lyric_{}.json".format(save_path, dissname), "w", encoding = "utf-8"), ensure_ascii = False)




import json
import re
import asyncio
import aiohttp
import time
from tqdm.asyncio import tqdm
import nest_asyncio
nest_asyncio.apply()


async def _async_get(url):
    session = aiohttp.ClientSession()
    response = await session.get(url)
    result = await response.text()
    await session.close()
    return result

async def _get_org_name_by_ripe(ip):
    api = "https://rest.db.ripe.net/search.json?source=ripe&query-string=%s" % ip # &source=apnic-grs
    res = await _async_get(api)

    try:
        json_res = json.loads(res)
        list_object = json_res["objects"]["object"]

        whois = {"ip": ip, "from": "RIPE NCC"}
        for ob in list_object:
            if ob["type"] == "organisation":
                list_attr = ob["attributes"]["attribute"]
                for attr in list_attr:
                    if attr["name"] == "org-name":
                        whois["org_name"] = attr["value"]
            elif ob["type"] == "inetnum":
                list_attr = ob["attributes"]["attribute"]
                for attr in list_attr:
                    if attr["name"] == "inetnum":
                        se = re.search("(.*?) - (.*)", attr["value"])
                        whois["start_address"] = se.group(1)
                        whois["end_address"] = se.group(2)
        assert "org_name" in whois and "start_address" in whois and "end_address" in whois
        return whois
    except Exception:
        return None


async def _get_org_name_by_arin(ip):
    api = "https://whois.arin.net/rest/ip/%s.json" % ip
    res = await _async_get(api)

    try:
        handle_json = json.loads(res)
        handle = handle_json["net"]["handle"]["$"]

        api2 = "https://whois.arin.net/rest/net/%s/pft.json?s=%s" % (handle, ip)
        res = await _async_get(api2)

        name = None
        start_address = None
        end_address = None
        json_whois = json.loads(res)["ns4:pft"]

        if "org" in json_whois:
            org = json_whois["org"]
            name = org["name"]["$"]
        if "customer" in json_whois:
            customer = json_whois["customer"]
            name = customer["name"]["$"]

        if "net" in json_whois:
            start_address = json_whois["net"]["startAddress"]["$"]
            end_address = json_whois["net"]["endAddress"]["$"]

        return {
            "ip": ip,
            "start_address": start_address,
            "end_address": end_address,
            "org_name": name,
            "from": "ARIN",
        }
    except Exception as e:
        return None


async def _get_org_name_by_lacnic(ip):
    api = "https://rdap.registro.br/ip/%s" % ip
    res = await _async_get(api)

    try:
        json_whois = json.loads(res)

        list_vcard = json_whois["entities"][0]["vcardArray"][1]
        for c in list_vcard:
            if c[0] == "fn":
                whois = c[3]
                whois["ip"] = ip
                whois["from"] = "LACNIC"
                return whois
    except Exception as e:
        return None


async def _get_org_name_by_apnic(ip):
    api = "http://wq.apnic.net/query?searchtext=%s" % ip
    res = await _async_get(api)

    try:
        json_whois = json.loads(res)
        for entry in json_whois:
            if entry["type"] == "object" and entry["objectType"] == "inetnum":
                attrs = entry["attributes"]
                for attr in attrs:
                    if attr["name"] == "descr":
                        whois = attr["values"][0]
                        whois["ip"] = ip
                        whois["from"] = "APNIC"
                        return whois
    except Exception:
        return None


async def _get_org_name(ip):
    org = await _get_org_name_by_arin(ip)
    # Asia Pacific Network Information Centre, South Brisbane #

    if org is not None and "Asia Pacific Network Information Centre" in org["org_name"]:
        org = await _get_org_name_by_apnic(ip)

    elif org is not None and "RIPE Network Coordination Centre" in org["org_name"]:
        org = await _get_org_name_by_ripe(ip)

    elif org is not None and "Latin American and Caribbean IP address Regional Registry" in org["org_name"]:
        org = await _get_org_name_by_lacnic(ip)

    elif org is None:
        return None

    # org = ner_tool.filter_out_company_char(org)

    return org


async def _get_org_names_for_list(ip_list, desc = "extracting org names"):
    # asyncio
    task_list = [_get_org_name(ip) for ip in ip_list]

    # # if use this code, rm the async and invoke this function directly (no progress)
    # t1 = time.time()
    # loop = asyncio.get_event_loop()
    # finished_tasks = loop.run_until_complete(asyncio.wait(task_list))
    # results = [t.result() for t in finished_tasks[0]]
    # loop.close()
    # t2 = time.time()
    # print("finished in {:.2} s.".format(t2 - t1))

    results = [await f for f in tqdm(asyncio.as_completed(task_list), desc = desc, total = len(task_list))]
    return results


def extract_org_names_friendly(ip_list, block_size = 100, sleep = 2):
    total_res = []
    loop = asyncio.get_event_loop()
    for start_id in range(0, len(ip_list), block_size):
        block = ip_list[start_id:(start_id+block_size)]
        desc = "extracting org names of ip block {}".format(start_id // block_size)
        results = loop.run_until_complete(_get_org_names_for_list(block, desc = desc))
        total_res.extend(results)
        time.sleep(sleep)
    # loop.close()
    return total_res


def extract_org_names_no_limited(ip_list):
    loop = asyncio.get_event_loop()
    results = loop.run_until_complete(_get_org_names_for_list(ip_list))
    # loop.close()
    return results


if __name__ == "__main__":
    # ip_list = ["154.17.24.36", "154.17.24.37", "154.17.24.39", "154.17.21.36"] * 100
    ip_list = ["54.83.30.211"] * 4
    # # friendly
    # res = extract_org_names_friendly(ip_list, block_size = 100, sleep = 2)

    # no limited
    res = extract_org_names_friendly(ip_list)
    print(res)



from Doraemon import investigation

# set arguments
key_word_list = ["山东华骜植化", "华泰证券", "广发证券", "海通证券"]  # organizations
api_key = "7dab451889b5f88db606750d615d9824"  # api key for captcha recognition, https://www.3023data.com/
save_dir = "./temp"  # path to save
# switch for websites
switch = {
    "baidu": True,  # 百度
    "neris": True,  # 中国证监会证券期货市场失信检索结果
    "mee_gov": True,  # 中华人民共和国生态环境部检索结果
    "mnr_gov": True,  # 中华人民共和国自然资源部检索结果
    "china_tax": True,  # 国家税务局重大税收违法案件信息公布栏检索结果
    "zxgk": True,  # 全国法院被执行人信息检索结果
}

# no need to change
investigator = investigation.OrgInvestigator(api_key)
investigator.investigate(key_word_list, save_dir, switch)
investigator.quite()



USER_AGENTS = [
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/20130406 Firefox/23.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Gecko/20100101 Firefox/18.0",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:39.0) Gecko/20100101 Firefox/39.0",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533+ (KHTML, like Gecko) Element Browser 5.0",
    "IBM WebExplorer /v0.94", "Galaxy/1.0 [en] (Mac OS X 10.5.6; U; en)",
    "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)",
    "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14",
    "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1468.0 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; TheWorld)",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36 OPR/26.0.1656.60",
    "Opera/8.0 (Windows NT 5.1; U; en)",
    "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.1) Gecko/20061208 Firefox/2.0.0 Opera 9.50",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.50",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:34.0) Gecko/20100101 Firefox/34.0",
    "Mozilla/5.0 (X11; U; Linux x86_64; zh-CN; rv:1.9.2.10) Gecko/20100922 Ubuntu/10.10 (maverick) Firefox/3.6.10",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.648.133 Safari/534.16",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.101 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko",
]
HEADERS = {
    "Content-Type": "*/*",
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "en-US,en;q=0.5",
    "Connection": "keep-alive",
    "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:39.0) Gecko/20100101 Firefox/39.0",
    "Referer": "https://www.google.com",
}

POOL = [
        "168.235.109.152:15028",
        "168.235.109.152:15027",
        "168.235.109.152:15026",
        "168.235.109.152:15025",
        "168.235.109.152:15024",
        "168.235.109.152:15023",
        "168.235.109.152:15022",
        "168.235.109.152:15021",
        "168.235.109.152:15020",
        "168.235.109.152:15019",
    ]


import re
import logging
from Doraemon.Requests import requests_dora, settings
import time
import requests
from bs4 import BeautifulSoup
from Doraemon.Requests import proxies_dora

def get_data5u_proxies(api=None):
    url = "http://api.ip.data5u.com/dynamic/get.html?order=53b3de376027aa3f699dc335d2bc0674&sep=3"
    if api is not None:
        url = api

    res = requests_dora.try_best_2_get(url)
    proxy = res.text.strip()

    if not re.match("\d+\.\d+\.\d+\.\d+:\d+", proxy):
        logging.warning("the proxy expired...")
        raise Exception

    return get_proxies(proxy)


def get_proxies(proxy):
    proxies = {
        "http": "http://{}".format(proxy),
        "https": "https://{}".format(proxy),
    }
    logging.info("current proxy: {}".format(proxy))
    return proxies


def set_pool(pool):
    '''
    should be invoked before get_proxies_fr_pool
    :param pool:
    :return:
    '''
    settings.POOL = pool


def get_proxies_fr_pool():
    if len(settings.POOL) == 0:
        logging.warning("the POOL is dry...")
        raise Exception
    pool = settings.POOL
    len_pool = len(pool)
    index = int(time.time() % len_pool)
    proxy = pool[index]
    return get_proxies(proxy)


def loc_proxy_ipv4(proxies):
    test_res = requests_dora.try_best_2_get("http://lumtest.com/myip.json", max_times=2, proxies=proxies, timeout=10)
    if test_res is not None and test_res.status_code == 200:
        return(test_res.text)
    else:
        logging.warning("failed to connect to geolocation url ...")
        return None


def loc_proxy(proxies):
    res = requests_dora.try_best_2_get("https://proxy6.net/en/myip", max_times=2, proxies=proxies, timeout=10)
    soup = BeautifulSoup(res.text, "lxml")
    # ip = soup.select_one("div.block-head > h1").get_text()
    loc_div = soup.select_one("div.myip-row > div")
    line_list = loc_div.select("dl")
    loc_info = {}
    for line in line_list:
        key = line.select_one("dt").get_text().strip().lower()
        val = line.select_one("dd").get_text().strip()
        loc_info[key] = val

    return loc_info


if __name__ == "__main__":
    proxies = get_proxies("mwFoRq:bKkLJT@91.188.243.238:9299")
    print(loc_proxy(proxies))
    pass



import logging
import random
import sys
import time
import requests
from Doraemon.Requests import settings


def get_default_headers():
    headers = settings.HEADERS
    ip_byte1 = random.randint(121, 255)
    ip_byte2 = random.randint(121, 255)
    ip_byte3 = random.randint(121, 255)
    ip_byte4 = random.randint(121, 255)
    headers["Referer"] = "{}.{}.{}.{}".format(ip_byte1, ip_byte2, ip_byte3, ip_byte4)
    return headers.copy()


def get_random_user_agent():
    random.seed(time.time())
    return random.choice(settings.USER_AGENTS)


def try_best_2_post(url, data=None, json=None, max_times=999, invoked_by=None, proxies=None, get_proxies_fun=None, **kwargs):
    '''
    :param url:
    :param data:
    :param json:
    :param max_times: max times for trying
    :param invoked_by: invoked by which function, used to debug
    :param get_proxies_fun: invoke the get_proxies_fun every request if it's been set
    :param proxies: it does not work if the get_proxies_fun has been set
    :param kwargs:
    :return:
    '''
    error_count = 0
    if invoked_by is None:
        invoked_by = "not set"
    while True:
        try:
            if get_proxies_fun:
                proxies = get_proxies_fun()
            res = requests.post(url, data=data, json=json, proxies=proxies, **kwargs)
            break
        except Exception as e:
            logging.warning("%s(invoked by: %s) go wrong..., attempt: %d" % (
            sys._getframe().f_code.co_name, invoked_by, error_count))
            logging.warning(e)
            random.seed(time.time())
            time.sleep(2 + 3 * random.random())
            error_count += 1
            if error_count > max_times:
                logging.warning("max error_count exceeded: %d" % (max_times))
                return None
    return res


def try_best_2_get(url, params=None, max_times=999, invoked_by=None, proxies=None, get_proxies_fun=None, **kwargs):
    '''
    :param url:
    :param params:
    :param max_times: max times for trying
    :param invoked_by: invoked by which function, used to debug
    :param get_proxies_fun: invoke the get_proxies_fun every request if it's been set
    :param proxies: it does not work if the get_proxies_fun has been set
    :param kwargs:
    :return:
    '''
    error_count = 0
    if invoked_by is None:
        invoked_by = "not set"
    while True:
        try:
            if get_proxies_fun:
                proxies = get_proxies_fun()
            res = requests.get(url, params=params, proxies=proxies, **kwargs)
            break
        except Exception as e:
            logging.warning("%s(invoked by: %s) go wrong..., attempt: %d" % (sys._getframe().f_code.co_name, invoked_by, error_count))
            logging.warning(e)
            random.seed(time.time())
            time.sleep(2 + 3 * random.random())
            error_count += 1
            if error_count > max_times:
                logging.warning("max error_count exceeded: %d" % (max_times))
                return None
    return res


if __name__ == "__main__":
    google_url = "https://www.google.com"
    proxy = "127.0.0.1:1080"
    proxies = {
        "https": "https://{}".format(proxy),
        "http": "http://{}".format(proxy),
    }
    print(try_best_2_get(google_url, max_times=3, timeout=5, headers=get_default_headers(), proxies=proxies).status_code)


# -*- coding: utf-8 -*-

from selenium import webdriver
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.common.by import By
from webdriver_manager.chrome import ChromeDriverManager
import time


class MyChrome(webdriver.Chrome):
    def __init__(self, headless=False, proxy=None, no_images=False, user_agent=None, cookie=None):
        '''
        :param headless: headless or not
        :param proxy: e.g. 127.0.0.1:1080
        :param no_images: set True if you don't wanna load images, this option is time-saving
        '''
        chrome_options = webdriver.ChromeOptions()
        if proxy is not None:
            chrome_options.add_argument('--proxy-server={}'.format(proxy))
        if headless:
            chrome_options.add_argument("--headless")
        if no_images:
            prefs = {'profile.managed_default_content_settings.images': 2}
            chrome_options.add_experimental_option('prefs', prefs)
        if user_agent is not None:
            chrome_options.add_argument('--user-agent={}'.format(user_agent))

        super(MyChrome, self).__init__(executable_path=ChromeDriverManager().install(),
                                       options=chrome_options)

    def wait_to_get_element(self, css_selector, timeout):
        while True:
            try:
                element = WebDriverWait(self, timeout).until(
                    ec.visibility_of_element_located((By.CSS_SELECTOR, css_selector)))
                break
            except Exception as e:
                print("page loading timeout! wait {} more seconds...".format(timeout))
                continue
        return element

    def screenshot_current_page(self, save_path):
        height = self.execute_script("return document.scrollingElement.scrollHeight;")
        width = self.execute_script("return document.scrollingElement.scrollWidth;")
        self.set_window_size(width, height)
        time.sleep(1)
        self.find_element_by_tag_name("body").screenshot(save_path)


if __name__ == "__main__":
    proxy = "127.0.0.1:1080"
    google_url = "https://www.google.com"
    baidu_url = "https://www.baidu.com"
    chrome = MyChrome(headless=True, proxy=proxy)
    chrome.get(google_url)
    print(chrome.page_source)


import html
import json
import math
import os
import re
from urllib import parse
from tqdm import tqdm
from Doraemon.Requests import requests_dora

KEY_LYRIC = "lyric"
KEY_CONTRIBUTORS = "contributors"
KEY_SINGER_NAME = "singer_name"
KEY_SINGER_ID = "singer_id"
KEY_SINGER_MID = "singer_mid"

def get_singer_list_page(area_id, page_ind):
    '''

    :param area_id: {'全部': -100, '内地': 200, '港台': 2, '欧美': 5, '日本': 4, '韩国': 3, '其他': 6}
    :param page_ind:
    :return:
    '''
    url = "https://u.y.qq.com/cgi-bin/musicu.fcg"
    header = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,  like Gecko) Chrome/59.0.3071.115 Safari/537.36",
        "cookie": "RK=7dNm4/X + Yj; tvfe_boss_uuid=bf00ee54e9081ab4; pgv_pvi=8772238336; pac_uid=1_857193777; pgv_pvid=6457341280; o_cookie=857193777; ptcz=c761e59c8c8d6bd5198866d02a5cb7313af1af468006c455d6c2b5d26201d42e; pgv_si=s10759168; _qpsvr_localtk=0.08285763449905015; ptisp=ctc; luin=o0857193777; lskey=00010000228dd1371b945c68ecfd3b71d3071425024a7a8a2a23e3ffcb5b9904c9f7088d2ea8c01539ffed92; pt2gguin=o0857193777; uin=o0857193777; skey=@Kydi7w0EI; p_uin=o0857193777; p_skey=HjsE9sEjznJfXk*9KFEeW4VZr6i3*tlXZ2nuzEw8kCg_; pt4_token=c-p6sv3JEboA51cSQ3ABqxM8O80Jct3jYYkgy-aEQuE_; p_luin=o0857193777; p_lskey=000400008f9c296cd10c03a5173d22a184aad124d791568e90e4198beb8ad699a4d02fbfc059f71ab3d8758c; ts_last=y.qq.com/portal/playlist.html; ts_refer=ui.ptlogin2.qq.com/cgi-bin/login; ts_uid=3392060960",
        "referer": "https://y.qq.com/portal/singer_list.html"
    }
    paramter = {
        "g_tk": "5381",
        "callback": "getUCGI9688380858412697",
        "jsonpCallback": "getUCGI9688380858412697",
        "loginUin": "0",
        "hostUin": "0",
        "format": "jsonp",
        "inCharset": "utf8",
        "outCharset": "utf-8",
        "notice": "0",
        "platform": "yqq",
        "needNewCode": "0",
        "data": '{"comm":{"ct":24,"cv":10000},"singerList":{"module":"Music.SingerListServer","method":"get_singer_list","param":{"area":%d,"sex":-100,"genre":-100,"index":-100,"sin":%d,"cur_page":%d}}}' % (area_id, (page_ind - 1) * 80, page_ind)
    }

    html_text = requests_dora.try_best_2_get(url=url, params=paramter, headers=header).text
    se = re.search("getUCGI9688380858412697\((.*)\)", html_text)
    json_str = None
    if se:
        json_str = se.group(1)

    data = json.loads(json_str)["singerList"]["data"]
    singerlist = data["singerlist"]
    try:
        area_ids = data["tags"]["area"]
    except Exception as e:
        area_ids = None
    singer_num_total = data["total"]
    return singerlist, singer_num_total, area_ids


def get_singer_list(area_id):
    singer_list_total = []
    singer_list, total, _ = get_singer_list_page(area_id, 1)
    singer_list_total.extend(singer_list)

    page_max = int(math.ceil(total / 80))
    for i in tqdm(range(1, page_max), desc = "getting singer list at area {}".format(area_id)):
        page_ind = i + 1
        singer_list, total, _ = get_singer_list_page(area_id, page_ind)
        singer_list_total.extend(singer_list)

    return singer_list_total


def crawl_song_list_page(singer_mid, begin):
    url = "https://c.y.qq.com/v8/fcg-bin/fcg_v8_singer_track_cp.fcg"
    header = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,  like Gecko) Chrome/59.0.3071.115 Safari/537.36",
        "cookie": "RK=7dNm4/X + Yj; tvfe_boss_uuid=bf00ee54e9081ab4; pgv_pvi=8772238336; pac_uid=1_857193777; pgv_pvid=6457341280; o_cookie=857193777; ptcz=c761e59c8c8d6bd5198866d02a5cb7313af1af468006c455d6c2b5d26201d42e; pgv_si=s10759168; _qpsvr_localtk=0.08285763449905015; ptisp=ctc; luin=o0857193777; lskey=00010000228dd1371b945c68ecfd3b71d3071425024a7a8a2a23e3ffcb5b9904c9f7088d2ea8c01539ffed92; pt2gguin=o0857193777; uin=o0857193777; skey=@Kydi7w0EI; p_uin=o0857193777; p_skey=HjsE9sEjznJfXk*9KFEeW4VZr6i3*tlXZ2nuzEw8kCg_; pt4_token=c-p6sv3JEboA51cSQ3ABqxM8O80Jct3jYYkgy-aEQuE_; p_luin=o0857193777; p_lskey=000400008f9c296cd10c03a5173d22a184aad124d791568e90e4198beb8ad699a4d02fbfc059f71ab3d8758c; ts_last=y.qq.com/portal/playlist.html; ts_refer=ui.ptlogin2.qq.com/cgi-bin/login; ts_uid=3392060960",
        "referer": "https://y.qq.com/n/yqq/singer/{}.html".format(singer_mid)
    }

    paramter = {
        "g_tk": "5381",
        "jsonpCallback": "MusicJsonCallbacksinger_track",
        "loginUin": "0",
        "hostUin": "0",
        "format": "jsonp",
        "inCharset": "utf8",
        "outCharset": "utf-8",
        "notice": "0",
        "platform": "yqq",
        "needNewCode": "0",
        "singermid": singer_mid,
        "order": "listen",
        "begin": begin,
        "num": "30",
        "songstatus": "1",
    }

    html_text = requests_dora.try_best_2_get(url=url, params=paramter, headers=header).text
    json_str = html_text.lstrip(" MusicJsonCallbacksinger_track(").rstrip(")").strip()
    data = json.loads(json_str)["data"]
    song_list = data["list"]
    total_num = data["total"]
    song_list_new = []
    for song in song_list:
        song = song["musicData"]
        song_new = {}
        song_new["albumid"] = song["albumid"]
        song_new["albummid"] = song["albummid"]
        song_new["albumname"] = song["albumname"]
        song_new["songid"] = song["songid"]
        song_new["songmid"] = song["songmid"]
        song_new["songname"] = song["songname"]
        song_list_new.append(song_new)
    return song_list_new, int(total_num)


def get_lyric(song):
    songid = song["songid"]
    songmid = song["songmid"]
    url = "https://c.y.qq.com/lyric/fcgi-bin/fcg_query_lyric.fcg"
    header = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,  like Gecko) Chrome/59.0.3071.115 Safari/537.36",
        "referer": "https://y.qq.com/n/yqq/song/{}.html".format(songmid)
    }
    paramters = {
        "nobase64": 1,
        "musicid": songid,
        "callback": "jsonp1",
        "g_tk": "1134533366",
        "jsonpCallback": "jsonp1",
        "loginUin": "0",
        "hostUin": "0",
        "format": "jsonp",
        "inCharset": "utf8",
        "outCharset": "utf-8",
        "notice": "0",
        "platform": "yqq",
        "needNewCode": "0"
    }
    html_text = requests_dora.try_best_2_get(url=url, params=paramters, headers=header).text
    res = json.loads(html_text.lstrip("jsonp1(").rstrip(")"))
    if "lyric" in res:
        lyric = res["lyric"]
        if "此歌曲为没有填词的纯音乐，请您欣赏" in lyric:
            return {}, []
        # decode
        lyric = html.unescape(lyric)
        lyric = html.unescape(lyric)
        lyric = parse.unquote(lyric)

        it = re.finditer(r"\[(\d+):(\d+.\d+)\](.+)", lyric)
        lyric_lines = []
        contributors_dict = {}
        for match in it:
            min = float(match.group(1))
            try:
                sec = float(match.group(2))
            except ValueError:
                sec = 0

            line = match.group(3)
            line = line.strip()
            if line == "":
                continue
            se_contributors = re.search("(.*?)[:：](.*)", line)
            if se_contributors:
                contributors_dict[se_contributors.group(1).strip()] = se_contributors.group(2).strip()
                continue
            lyric_lines.append({
                "time": min * 60 + sec,
                "line": line,
            })

        return contributors_dict, lyric_lines[1:]
    else:
        return {}, []


def crawl_song_list(singer, area):
    singer_mid = singer[KEY_SINGER_MID]
    singer_name = singer[KEY_SINGER_NAME]
    song_list_total = []
    song_list, total = crawl_song_list_page(singer_mid, 0)
    song_list_total.extend(song_list)

    p_ind_list = range(30, total, 30)
    for i in tqdm(p_ind_list, desc="getting song list of {}@{}...".format(singer_name, area)):
        song_list, total = crawl_song_list_page(singer_mid, i)
        song_list_total.extend(song_list)

    for song in tqdm(song_list_total, desc="getting the lyrics of the songs of {}@{}...".format(singer_name, area)):
        contributors, lyric = get_lyric(song)
        song[KEY_CONTRIBUTORS] = contributors
        song[KEY_LYRIC] = lyric
        song[KEY_SINGER_NAME] = singer_name

    return song_list_total


def crawl_songs(area_list, save_path):
    # get progress
    singer_id_done = []
    for root, dirs, files in os.walk(save_path):
        for file_name in files:
            singer_id = re.search("song_list_.*_(.*).json", file_name).group(1)
            singer_id_done.append(int(singer_id))

    _, _, area_ids = get_singer_list_page(-100, 1)
    area_2_id = {}
    for item in area_ids:
        area_2_id[item["name"]] = item["id"]

    for area in area_list:
        singer_list = get_singer_list(area_2_id[area])
        for singer in singer_list:
            singer_name = singer[KEY_SINGER_NAME]
            singer_id = singer[KEY_SINGER_ID]
            if singer_id in singer_id_done:
                continue
            song_list = crawl_song_list(singer, area)
            json.dump(song_list,
                      open("%s/song_list_%s_%s.json" % (save_path, singer_name, singer_id), "w", encoding="utf-8"), ensure_ascii = False)

if __name__ == "__main__":
    # # get singer list
    # singers = get_singer_list(-100)
    # print(len(singers))

    # crawl songs by areas
    area_list = ["港台", "内地"] # {'全部': -100, '内地': 200, '港台': 2, '欧美': 5, '日本': 4, '韩国': 3, '其他': 6}
    save_path = "./qq_music_songs_by_area"
    if not os.path.exists(save_path):
        os.makedirs(save_path)
    crawl_songs(area_list, save_path)




# -*- coding:utf-8 -*-
import requests
import re
import execjs
import json
import logging
import html
import time
from tqdm import tqdm
from Doraemon.Requests import requests_dora

'''reference: language 2 code

{code:'auto',name:'检测语言'},{code:'sq',name:'阿尔巴尼亚语'},{code:'ar',name:'阿拉伯语'},{code:'am',name:'阿姆哈拉语'},{code:'az',name:'阿塞拜疆语'},{code:'ga',name:'爱尔兰语'},{code:'et',name:'爱沙尼亚语'},{code:'eu',name:'巴斯克语'},{code:'be',name:'白俄罗斯语'},{code:'bg',name:'保加利亚语'},{code:'is',name:'冰岛语'},{code:'pl',name:'波兰语'},{code:'bs',name:'波斯尼亚语'},{code:'fa',name:'波斯语'},{code:'af',name:'布尔语(南非荷兰语)'},{code:'da',name:'丹麦语'},{code:'de',name:'德语'},{code:'ru',name:'俄语'},{code:'fr',name:'法语'},{code:'tl',name:'菲律宾语'},{code:'fi',name:'芬兰语'},{code:'fy',name:'弗里西语'},{code:'km',name:'高棉语'},{code:'ka',name:'格鲁吉亚语'},{code:'gu',name:'古吉拉特语'},{code:'kk',name:'哈萨克语'},{code:'ht',name:'海地克里奥尔语'},{code:'ko',name:'韩语'},{code:'ha',name:'豪萨语'},{code:'nl',name:'荷兰语'},{code:'ky',name:'吉尔吉斯语'},{code:'gl',name:'加利西亚语'},{code:'ca',name:'加泰罗尼亚语'},{code:'cs',name:'捷克语'},{code:'kn',name:'卡纳达语'},{code:'co',name:'科西嘉语'},{code:'hr',name:'克罗地亚语'},{code:'ku',name:'库尔德语'},{code:'la',name:'拉丁语'},{code:'lv',name:'拉脱维亚语'},{code:'lo',name:'老挝语'},{code:'lt',name:'立陶宛语'},{code:'lb',name:'卢森堡语'},{code:'ro',name:'罗马尼亚语'},{code:'mg',name:'马尔加什语'},{code:'mt',name:'马耳他语'},{code:'mr',name:'马拉地语'},{code:'ml',name:'马拉雅拉姆语'},{code:'ms',name:'马来语'},{code:'mk',name:'马其顿语'},{code:'mi',name:'毛利语'},{code:'mn',name:'蒙古语'},{code:'bn',name:'孟加拉语'},{code:'my',name:'缅甸语'},{code:'hmn',name:'苗语'},{code:'xh',name:'南非科萨语'},{code:'zu',name:'南非祖鲁语'},{code:'ne',name:'尼泊尔语'},{code:'no',name:'挪威语'},{code:'pa',name:'旁遮普语'},{code:'pt',name:'葡萄牙语'},{code:'ps',name:'普什图语'},{code:'ny',name:'齐切瓦语'},{code:'ja',name:'日语'},{code:'sv',name:'瑞典语'},{code:'sm',name:'萨摩亚语'},{code:'sr',name:'塞尔维亚语'},{code:'st',name:'塞索托语'},{code:'si',name:'僧伽罗语'},{code:'eo',name:'世界语'},{code:'sk',name:'斯洛伐克语'},{code:'sl',name:'斯洛文尼亚语'},{code:'sw',name:'斯瓦希里语'},{code:'gd',name:'苏格兰盖尔语'},{code:'ceb',name:'宿务语'},{code:'so',name:'索马里语'},{code:'tg',name:'塔吉克语'},{code:'te',name:'泰卢固语'},{code:'ta',name:'泰米尔语'},{code:'th',name:'泰语'},{code:'tr',name:'土耳其语'},{code:'cy',name:'威尔士语'},{code:'ur',name:'乌尔都语'},{code:'uk',name:'乌克兰语'},{code:'uz',name:'乌兹别克语'},{code:'es',name:'西班牙语'},{code:'iw',name:'希伯来语'},{code:'el',name:'希腊语'},{code:'haw',name:'夏威夷语'},{code:'sd',name:'信德语'},{code:'hu',name:'匈牙利语'},{code:'sn',name:'修纳语'},{code:'hy',name:'亚美尼亚语'},{code:'ig',name:'伊博语'},{code:'it',name:'意大利语'},{code:'yi',name:'意第绪语'},{code:'hi',name:'印地语'},{code:'su',name:'印尼巽他语'},{code:'id',name:'印尼语'},{code:'jw',name:'印尼爪哇语'},{code:'en',name:'英语'},{code:'yo',name:'约鲁巴语'},{code:'vi',name:'越南语'},{code:'zh-CN',name:'中文'}],target_code_name:[{code:'sq',name:'阿尔巴尼亚语'},{code:'ar',name:'阿拉伯语'},{code:'am',name:'阿姆哈拉语'},{code:'az',name:'阿塞拜疆语'},{code:'ga',name:'爱尔兰语'},{code:'et',name:'爱沙尼亚语'},{code:'eu',name:'巴斯克语'},{code:'be',name:'白俄罗斯语'},{code:'bg',name:'保加利亚语'},{code:'is',name:'冰岛语'},{code:'pl',name:'波兰语'},{code:'bs',name:'波斯尼亚语'},{code:'fa',name:'波斯语'},{code:'af',name:'布尔语(南非荷兰语)'},{code:'da',name:'丹麦语'},{code:'de',name:'德语'},{code:'ru',name:'俄语'},{code:'fr',name:'法语'},{code:'tl',name:'菲律宾语'},{code:'fi',name:'芬兰语'},{code:'fy',name:'弗里西语'},{code:'km',name:'高棉语'},{code:'ka',name:'格鲁吉亚语'},{code:'gu',name:'古吉拉特语'},{code:'kk',name:'哈萨克语'},{code:'ht',name:'海地克里奥尔语'},{code:'ko',name:'韩语'},{code:'ha',name:'豪萨语'},{code:'nl',name:'荷兰语'},{code:'ky',name:'吉尔吉斯语'},{code:'gl',name:'加利西亚语'},{code:'ca',name:'加泰罗尼亚语'},{code:'cs',name:'捷克语'},{code:'kn',name:'卡纳达语'},{code:'co',name:'科西嘉语'},{code:'hr',name:'克罗地亚语'},{code:'ku',name:'库尔德语'},{code:'la',name:'拉丁语'},{code:'lv',name:'拉脱维亚语'},{code:'lo',name:'老挝语'},{code:'lt',name:'立陶宛语'},{code:'lb',name:'卢森堡语'},{code:'ro',name:'罗马尼亚语'},{code:'mg',name:'马尔加什语'},{code:'mt',name:'马耳他语'},{code:'mr',name:'马拉地语'},{code:'ml',name:'马拉雅拉姆语'},{code:'ms',name:'马来语'},{code:'mk',name:'马其顿语'},{code:'mi',name:'毛利语'},{code:'mn',name:'蒙古语'},{code:'bn',name:'孟加拉语'},{code:'my',name:'缅甸语'},{code:'hmn',name:'苗语'},{code:'xh',name:'南非科萨语'},{code:'zu',name:'南非祖鲁语'},{code:'ne',name:'尼泊尔语'},{code:'no',name:'挪威语'},{code:'pa',name:'旁遮普语'},{code:'pt',name:'葡萄牙语'},{code:'ps',name:'普什图语'},{code:'ny',name:'齐切瓦语'},{code:'ja',name:'日语'},{code:'sv',name:'瑞典语'},{code:'sm',name:'萨摩亚语'},{code:'sr',name:'塞尔维亚语'},{code:'st',name:'塞索托语'},{code:'si',name:'僧伽罗语'},{code:'eo',name:'世界语'},{code:'sk',name:'斯洛伐克语'},{code:'sl',name:'斯洛文尼亚语'},{code:'sw',name:'斯瓦希里语'},{code:'gd',name:'苏格兰盖尔语'},{code:'ceb',name:'宿务语'},{code:'so',name:'索马里语'},{code:'tg',name:'塔吉克语'},{code:'te',name:'泰卢固语'},{code:'ta',name:'泰米尔语'},{code:'th',name:'泰语'},{code:'tr',name:'土耳其语'},{code:'cy',name:'威尔士语'},{code:'ur',name:'乌尔都语'},{code:'uk',name:'乌克兰语'},{code:'uz',name:'乌兹别克语'},{code:'es',name:'西班牙语'},{code:'iw',name:'希伯来语'},{code:'el',name:'希腊语'},{code:'haw',name:'夏威夷语'},{code:'sd',name:'信德语'},{code:'hu',name:'匈牙利语'},{code:'sn',name:'修纳语'},{code:'hy',name:'亚美尼亚语'},{code:'ig',name:'伊博语'},{code:'it',name:'意大利语'},{code:'yi',name:'意第绪语'},{code:'hi',name:'印地语'},{code:'su',name:'印尼巽他语'},{code:'id',name:'印尼语'},{code:'jw',name:'印尼爪哇语'},{code:'en',name:'英语'},{code:'yo',name:'约鲁巴语'},{code:'vi',name:'越南语'},{code:'zh-TW',name:'中文(繁体)'},{code:'zh-CN',name:'中文(简体)'}
'''


js = '''
    function b(a, b) {
        for (var d = 0; d < b.length - 2; d += 3) {
            var c = b.charAt(d + 2),
                c = "a" <= c ? c.charCodeAt(0) - 87 : Number(c),
                c = "+" == b.charAt(d + 1) ? a >>> c : a << c;
            a = "+" == b.charAt(d) ? a + c & 4294967295 : a ^ c
        }
        return a
    }

    function tk(a,TKK) {
        for (var e = TKK.split("."), h = Number(e[0]) || 0, g = [], d = 0, f = 0; f < a.length; f++) {
            var c = a.charCodeAt(f);
            128 > c ? g[d++] = c : (2048 > c ? g[d++] = c >> 6 | 192 : (55296 == (c & 64512) && f + 1 < a.length && 56320 == (a.charCodeAt(f + 1) & 64512) ? (c = 65536 + ((c & 1023) << 10) + (a.charCodeAt(++f) & 1023), g[d++] = c >> 18 | 240, g[d++] = c >> 12 & 63 | 128) : g[d++] = c >> 12 | 224, g[d++] = c >> 6 & 63 | 128), g[d++] = c & 63 | 128)
        }
        a = h;
        for (d = 0; d < g.length; d++) a += g[d], a = b(a, "+-a^+6");
        a = b(a, "+-3^+b+-f");
        a ^= Number(e[1]) || 0;
        0 > a && (a = (a & 2147483647) + 2147483648);
        a %= 1E6;
        return a.toString() + "." + (a ^ h)
    }
    '''
tk_calculator = execjs.compile(js)  # use to compute the token for translating request


def get_TKK(get_proxies_fun=None):
    '''
    to get a value for calculating the token for constructing translating request
    :return: TKK
    '''
    url = "https://translate.google.cn"
    logging.warning("start getting tkk...")

    res = requests_dora.try_best_2_get(url, get_proxies_fun=get_proxies_fun)

    TKK = re.search("tkk:'(.*?)'", res.text).group(1)
    return TKK


def trans_req(ori_text, sl="auto", tl="en", get_proxies_fun=None):
    '''
    construct translating request
    :param ori_text:
    :param sl:
    :param tl:
    :return:
    '''
    ori_text = html.unescape(ori_text)  # unescape the html
    res = None

    while True:
        try:
            tk = tk_calculator.call("tk", ori_text, get_TKK())

            url_trans = "https://translate.google.cn/translate_a/single"

            payload = {
                "client": "t",
                "sl": sl,
                "tl": tl,
                "dt": "t",
                "ie": "UTF-8",
                "oe": "UTF-8",
                "otf": "1",
                "ssel": "0",
                "tsel": "0",
                "kc": "1",
                "tk": tk,
                "q": ori_text,
            }

            # res = req_t_death("POST", url_trans, payload)
            res = requests_dora.try_best_2_post(url_trans, data=payload, get_proxies_fun=get_proxies_fun)
            # print(res.text)
        except Exception as e:
            logging.warning(e)
            logging.warning("error, waiting and try again...")
            # logging.warning("text: %s " % ori_text)
            time.sleep(1)
            continue

        if res.status_code == 200:
            break

    js = None
    try:
        js = json.loads(res.text)
    except Exception as e:
        logging.warning(e)
        return []

    return js


def get_language_type(ori_text, get_proxies_fun=None):
    '''
    identify which language it is
    :param ori_text:
    :return:
    '''
    info = ori_text[:30] if len(ori_text) > 30 else ori_text
    logging.warning("start identifying: {}...".format(info))
    js = trans_req(ori_text, get_proxies_fun)
    return js[2]


def trans(ori_text, sl="auto", tl="en", get_proxies_fun=None):
    '''
    translate text of which the length is less than 5000
    :param ori_text:
    :param sl:
    :param tl:
    :return:
    '''
    info = ori_text[:50] if len(ori_text) > 50 else ori_text
    logging.warning("start translating: {}...".format(info))

    js = trans_req(ori_text, sl, tl, get_proxies_fun)
    trans_text = ""
    if js[2] == "en":
        trans_text = js[0][0][0]
    else:
        for pas in js[0]:
            trans_text += pas[0]

    info = trans_text[:50] if len(trans_text) > 50 else trans_text
    logging.warning("translation result: {}...".format(info))
    return trans_text


def trans_long(ori_text, sl="auto", tl="en", get_proxies_fun=None):
    '''
    split the long text into pieces and translate

    :param ori_text: text whose len > 5000
    :param sl: source language
    :param tl: target language
    :return:
    '''
    stop_char = ["。", ".", ]
    start_flag = 0
    split_marks = []

    pointor = start_flag + 5000 + 1

    while True:
        while ori_text[pointor] not in stop_char:
            pointor -= 1
        split_marks.append(pointor)
        pointor += 5000
        if pointor >= len(ori_text):
            break

    snippets = []
    start_flag = 0
    for m in split_marks:
        snippets.append(ori_text[start_flag:(m + 1)])
        start_flag = m + 1

    if split_marks[-1] != len(ori_text) - 1:
        snippets.append(ori_text[start_flag:len(ori_text)])

    if get_language_type(snippets[0]) == tl:
        return ori_text

    en_text = ""
    for sni in tqdm(snippets):
        en_text += trans(sni, sl, tl, get_proxies_fun)

    return en_text


if __name__ == "__main__":
    def get_proxy():
        # proxy_str = requests.get(
        #     "http://api.ip.data5u.com/dynamic/get.html?order=53b3de376027aa3f699dc335d2bc0674&sep=3").text.strip()
        # proxies = {"http": "http://%s" % proxy_str,
        #            "https": "http://%s" % proxy_str, }
        proxy_str = "127.0.0.1:1080"
        proxies = {"http": "http://%s" % proxy_str,
                   "https": "http://%s" % proxy_str, }
        return proxies

    res = trans("中华人民共和国", get_proxies_fun=get_proxy)
    print(res)



import binascii
import time
from urllib import parse
import requests
import re
import json
import logging
import base64
import random
import rsa
import logging
import getopt


class WeiboRobot:

    def __init__(self, username, password):
        self.__session = requests.session()
        self.__uid = ""
        self.__user_nick = ""

        login_info = self.__login_simulate(username, password)
        logging.warning("login info: {}".format(login_info))
        assert login_info is not None
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8",
            "Accept-Encoding": "gzip,deflate,br",
            "Accept-Language": "zh-CN,zh;q=0.8",
            "Connection": "keep-alive",
            "Host": "weibo.com",
            "Referer": "https://weibo.com/",
            "Upgrade-Insecure - Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.86 Safari/537.36",
        }

        self.__session.headers.update(headers)

    def __prelogin(self, username_encoded):
        url = "http://login.sina.com.cn/sso/prelogin.php"
        payload = {
            "entry": "weibo",
            "callback": "sinaSSOController.preloginCallBack",
            "su": username_encoded,
            "rsakt": "mod",
            "checkpin": "1",
            "client": "ssologin.js(v1.4.19)",
            "_": int(time.time() * 1000),
        }
        res = self.__session.post(url, data=payload, )
        prelogin_json = re.search("\((.*)\)", res.text).group(1)

        return json.loads(prelogin_json)

    def __get_username(self, username):
        return base64.b64encode(parse.quote_plus(username).encode("utf-8"))

    def __get_password(self, pass_word, servertime, nonce, pubkey):
        string = (str(servertime) + "\t" + str(nonce) + "\n" + str(pass_word)).encode("utf-8")
        public_key = rsa.PublicKey(int(pubkey, 16), int("10001", 16))
        password = rsa.encrypt(string, public_key)
        password = binascii.b2a_hex(password)
        return password.decode()

    def __identify_captcha(self, cap_b64):
        typeid = 35
        appkey = "f1d9b361016be2d78f0684fb5891f2c3"
        url = "https://way.jd.com/showapi/checkcode_ys?typeId=%d&convert_to_jpg=0&appkey=%s" % (typeid, appkey)

        payload = {
            "body": "img_base64=%s" % cap_b64,
        }
        res = requests.post(url, data=payload)
        res_js = res.json()
        if res_js["code"] == "10000" and res_js["result"]["showapi_res_code"] == 0 and \
                        res_js["result"]["showapi_res_body"]["ret_code"] == 0:
            return res_js["result"]["showapi_res_body"]["Result"]
        else:
            logging.warning(res_js)
            return None

    def __login_request(self, username_encoded, pw_encrypted, prelogin_json):
        url_login = "http://login.sina.com.cn/sso/login.php?client=ssologin.js(v1.4.19)&wsseretry=servertime_error"
        payload = {
            "entry": "weibo",
            "gateway": "1",
            "from": "",
            "savestate": "7",
            "userticket": "1",
            "vsnf": "1",
            "service": "miniblog",
            "encoding": "UTF-8",
            "pwencode": "rsa2",
            "sr": "1280*800",
            "prelt": "529",
            "url": "http://weibo.com/ajaxlogin.php?framelogin=1&callback=parent.sinaSSOController.feedBackUrlCallBack",
            "rsakv": prelogin_json["rsakv"],
            "servertime": prelogin_json["servertime"],
            "nonce": prelogin_json["nonce"],
            "su": username_encoded,
            "sp": pw_encrypted,
            "returntype": "TEXT",
        }

        # 判断验证码
        json_ticket = None
        if "showpin" in prelogin_json and prelogin_json["showpin"] == 1:  # 如果需要输入验证码
            payload["pcid"] = prelogin_json["pcid"]

            url_pin = "http://login.sina.com.cn/cgi/pin.php?r=%d&s=0&p=%s" % (
                int(time.time() * 1000), prelogin_json["pcid"])  # 获得验证码的地址

            res = self.__session.post(url_pin, data=payload)
            captcha_img = open("./captcha.png", "wb")
            captcha_img.write(res.content)
            captcha_img.close()

            captcha_img_b64 = base64.b64encode(res.content).decode()
            iden = self.__identify_captcha(captcha_img_b64)

            code = ""
            if iden is None:
                code = input("请输入验证码:")
            else:
                code = iden

            payload["door"] = code

        # login weibo.com
        res = self.__session.post(url_login, data=payload)
        json_ticket = res.json()
        if json_ticket["retcode"] != "0":
            logging.warning("WeiBoLogin failed: %s", json_ticket)
            return False
        else:
            params = {
                "callback": "sinaSSOController.callbackLoginStatus",
                "client": "ssologin.js(v1.4.19)",
                "ticket": json_ticket["ticket"],
                "ssosavestate": int(time.time()),
                "_": int(time.time() * 1000),
            }
            response = self.__session.get("http://passport.weibo.com/wbsso/login", params=params)
            json_data_2 = json.loads(re.search(r"\((?P<result>.*)\)", response.text).group("result"))
            if json_data_2["result"] is True:
                self.__uid = json_data_2["userinfo"]["uniqueid"]
                self.__user_nick = json_data_2["userinfo"]["displayname"]
                logging.warning("WeiBoLogin succeed: %s", json_data_2)
                return True
            else:
                logging.warning("WeiBoLogin failed: %s", json_data_2)
                return False

    def __is_home(self):
        res = self.__session.get("https://weibo.com/u/%s/home" % (self.__uid)).text
        if u"我的首页" in res:
            return True
        else:
            return False

    def __login_simulate(self, username, pw):
        max_times = 5
        count_login = 0
        while True:
            username_encoded = self.__get_username(username)
            prelogin_json = self.__prelogin(username_encoded)
            pw_encrypted = self.__get_password(pw, prelogin_json["servertime"], prelogin_json["nonce"],
                                               prelogin_json["pubkey"])

            login_res = self.__login_request(username_encoded, pw_encrypted, prelogin_json)
            if login_res and self.__is_home():  # 一旦登录成功就返回user
                return {"uid": self.__uid, "session": self.__session}  # 返回登录的user的dict
            else:
                count_login += 1
                if count_login == max_times:
                    logging.warning("exceeded the max times of login...")
                    return None
                logging.warning("%s login failed, try again..." % username)
                random.seed(time.time())
                sleeptime = 5 * random.random()
                logging.warning("sleep %ds" % sleeptime)
                time.sleep(sleeptime)

    # api for users
    # ------------------------------------------------------------------------------------------------------
    def edit_profile(self):
        '''
        edit profile
        :return:
        '''
        payload = {
            'setting_rid': 'd1Vlf235Q-LBj/DshHM8stx-1GQ=',
            'nickname': '睡小迪_爱琪琪阿四',
            'realname': '薛梦雪',
            'gender': 'f',
            'sextrend[]': '0',
            'blog': '',
            'mydesc': '一心一意',
            'province': '11',
            'city': '1',
            'love': '',
            'Date_Year': '1998',
            'birthday_m': '03',
            'birthday_d': '06',
            'blood': '',
            'pub_name': '0',
            'pub_sextrend': '0',
            'pub_love': '1',
            'pub_birthday': '3',
            'pub_blood': '1',
            'pub_blog': '2',
        }
        res = self.__session.post("https://account.weibo.com/set/aj/iframe/editinfo",
                                  headers={"Host": "account.weibo.com", }, data=payload).text
        if res['code'] == '100000':
            print("edit_profile success!")
            return True
        else:
            print("edit_profile fail... " + res['msg'])
            return False

    def edit_edu(self):
        '''
        edit education
        :return:
        '''
        payload = {'name': '北京外国语大学',
                   'school_type': '1',
                   'start': '2014',
                   'privacy': '2',
                   'school_province': '11',
                   'school_id': '243973',
                   }

        res = self.__session.post("https://account.weibo.com/set/aj/iframe/edupost",
                                  headers={"Host": "account.weibo.com", }, data=payload).text
        try:
            res = json.loads(res)
            if res['code'] == '100000':
                print("edit_edu success!")
                return True
            else:
                print("edit_edu fail... " + res['msg'])
                return False
        except:
            print(res)
            return False

    def post(self, text, img_url_list=None, img_id_list=None):
        '''
        post text
        :param text:
        :return:
        '''
        payload = {
            'text': text,
            'appkey': '',
            'style_type': '1',
            'pic_id': '',
            'tid': '',
            'pdetail': '',
            'rank': '0',
            'rankid': '',
            'module': 'stissue',
            'pub_source': 'main_',
            'pub_type': 'dialog',
            'isPri': '0',
            '_t': '0',
        }

        if img_id_list is None and img_url_list is not None:
            img_id_list = []
            for url in img_url_list:
                img_id = self.__up_img(url)
                if img_id is not None:
                    img_id_list.append(img_id)

        if img_id_list is not None:
            pic_id = "|".join(img_id_list)
            payload["pic_id"] = pic_id

        res = self.__session.post("https://weibo.com/aj/mblog/add", data=payload).text
        res = json.loads(res)
        if res['code'] == '100000':
            logging.warning("post blog success!")
            return True
        else:
            logging.warning("post blog fail... " + res['msg'])
            return False

    def like_blog(self, blog_mid):
        '''
        like a blog
        :param blog_mid:
        :return:
        '''
        payload = {
            'version': ' mini',
            'qid': ' heart',
            'mid': blog_mid,
            'loc': ' profile',
            'cuslike': ' 1',
            '_t': ' 0',
        }
        res = self.__session.post("https://weibo.com/aj/v6/like/add", data=payload).text

        try:
            res = json.loads(res)
            if res['code'] == '100000':
                logging.warning("%s like_blog %s success!" % (self.__uid, blog_mid))
                return True
            else:
                logging.warning("%s like_blog %s fail... msg: %s" % (self.__uid, blog_mid, res['msg']))
                return False
        except:
            logging.warning("%s like_blog %s fail... msg: %s" % (self.__uid, blog_mid, res))
            return False

    #
    def like_object(self, ob_id, ob_type):
        payload = {
            'object_id': ob_id,
            'object_type': ob_type,
            '_t': '0',
        }
        res = self.__session.post("https://weibo.com/aj/v6/like/objectlike", data=payload).text
        res = json.loads(res)
        if res['code'] == '100000':
            logging.warning("like_object(%s) success!" % (ob_type))
            return True
        else:
            logging.warning("like_object(%s) fail... " % (ob_type) + res['msg'])
            return False

    def like_comment(self, comment_id):
        '''
        like a comment
        :param comment_id:
        :return:
        '''
        self.like_object(comment_id, "comment")

    # 评论和转发
    def comment_forward(self, blog_mid, text, forward=False, img_url=None, img_id=None):  # 目标微博id, 评论者的uid，评论内容，forward = {"0", "1"}是否转发
        '''
        send a comment and forward it
        :param blog_mid: the id of the target comment
        :param text: what you wanna send
        :param forward: forward it or not
        :param img_url: set the url of a image if you wanna send one
        :param img_id: set if you wanna send an existing image(if you have upload an image to the Weibo,
                there would be an id for it
        :return:
        '''
        payload = {
            'act': 'post',
            'mid': blog_mid,
            'uid': self.__uid,
            'isroot': '0',
            'content': text,
            'module': 'scommlist',
            'group_source': '',
            '_t': '0',
            "forward": "1" if forward else "0",
        }

        if img_url is not None:
            pic_id = self.__up_img(img_url)
            payload["pic_id"] = pic_id
        if img_id is not None:
            payload["pic_id"] = img_id

        res = self.__session.post("https://weibo.com/aj/v6/comment/add", data=payload)

        try:
            res = json.loads(res.text)
            if res['code'] == '100000':
                logging.warning("comment_forward success! comments: %s" % text)
            else:
                logging.warning("%s comment_forward %s fail... code: %s msg: %s" % (
                    self.__uid, blog_mid, res['code'], res['msg']))
        except Exception as e:
            logging.warning(e)

        return res

    def reply_comment(self, blog_mid, comment_id, text, approval=False
                      , forward=False, img_url=None, img_id=None):
        '''
        reply a comment, the args are the same as function comment_forward
        :param blog_mid:
        :param comment_id:
        :param text:
        :param approval:
        :param forward:
        :param img_url:
        :param img_id:
        :return:
        '''
        payload = {
            "act": "reply",
            "canUploadImage": "1",
            "cid": comment_id,
            "content": text,
            "dissDataFromFeed": "[object Object]",
            "ispower": "1",
            "isroot": "0",
            "approvalComment": "true" if approval else "false",
            # "location": "page_100505_home",
            "forward": "1" if forward else "0",
            "mid": blog_mid,
            "module": "scommlist",
            # "nick": "一拳超超",
            # "ouid": "2887964854",
            # "pdetail": "1005056219737121",
            # "root_comment_id": "4221420517644305",
            # "status_owner_user": "6219737121",
            "uid": self.__uid,
        }

        if img_url is not None:
            pic_id = self.__up_img(img_url)
            payload["pic_id"] = pic_id
        if img_id is not None:
            payload["pic_id"] = img_id

        res = self.__session.post("https://weibo.com/aj/v6/comment/add", data=payload)

        try:
            res = json.loads(res.text)
            if res['code'] == '100000':
                logging.warning("comment_forward success! comments: %s" % text)
            else:
                logging.warning(
                    "%s comment_forward %s fail... code: %s msg: %s" % (
                        self.__uid, comment_id, res['code'], res['msg']))
        except Exception as e:
            print("it is not json")
            logging.warning(e)

        return res

    def __img_bs64(self, url):
        '''
        image to base64 encoding
        :param url:
        :return:
        '''
        ls_f = ""
        if "http" in url:  # 网络地址
            content = requests.get(url).content
            ls_f = base64.b64encode(content)  # 读取文件内容，转换为base64编码

        else:  # 本地地址
            f = open(url, 'rb')  # 二进制方式打开图文件
            ls_f = base64.b64encode(f.read())  # 读取文件内容，转换为base64编码
            f.close()
        img_bs64_str = ls_f.decode()  # 解码成字符串

        return img_bs64_str

    def __up_img(self, img_url):
        '''
        upload img and get img id
        :param img_url:
        :return:
        '''
        img_bs64_str = self.__img_bs64(img_url)

        url = "https://picupload.weibo.com/interface/pic_upload.php"
        payload = {
            "cb": "https://weibo.com /aj/static/upimgback.html?_wv=5&callback=STK_ijax_151694642763479",
            "mime": "image/jpeg",
            "data": "base64",
            "url": "weibo.com/u/%s" % self.__uid,
            "markpos": "1",
            "logo": "1",
            # "nick": "@王雨城_Vision_Wong",
            "marks": "1",
            "app": "miniblog",
            "s": "rdxt",
            "file_source": "3",
        }

        img = {
            "b64_data": img_bs64_str,
        }
        try:
            res = self.__session.post(url, data=payload, files=img)
        except Exception as e:
            se = re.search("&pid=([\S]*)", str(e))
            if se is None:
                return None
            img_id = se.group(1)
            return img_id

    def del_blog(self, blog_mid):
        '''
        delete a blog
        :param blog_mid:
        :return:
        '''
        payload = {
            'mid': blog_mid,
        }
        res = self.__session.post("https://weibo.com/aj/mblog/del", data=payload).text
        res = json.loads(res)
        if res['code'] == '100000':
            print("mblog %s del_blog success!" % blog_mid)
            return True
        else:
            print("mblog %s del_blog fail... " % blog_mid + res['msg'])
            return False

    def del_comment(self, blog_mid, comment_id):
        '''
        delete a comment under your post
        :param blog_mid:
        :param comment_id:
        :return:
        '''
        payload = {
            'act': 'delete',
            'mid': blog_mid,
            'cid': comment_id,
            'uid': self.__uid,
            'is_block': '0',
            'rid': comment_id,
            'oid': '',
            '_t': '0',
        }
        res = self.__session.post("https://weibo.com/aj/comment/del", data=payload).text
        res = json.loads(res)
        if res['code'] == '100000':
            print("del_comment success!")
            return True
        else:
            print("del_comment fail... " + res['msg'])
            return False

    def follow_unfo(self, object_uid, follow):
        '''
        follow or unfollow someone
        :param object_uid:
        :param follow: boolean
        :return:
        '''
        payload = {
            'uid': object_uid,
            'refer_flag': '1005050002',
            'oid': object_uid,
            'wforce': '1',
            'nogroup': 'false',
            'refer_from': 'profile_headerv6',
            '_t': '0',
        }
        follow = "followed" if follow == "1" else "unfollow"

        res = self.__session.post("https://weibo.com/aj/f/%s" % (follow), data=payload).text
        res = json.loads(res)
        if res['code'] == '100000':
            print("follow_unfo success!")
            return True
        else:
            print("follow_unfo fail... " + res['msg'])
            return False

    def home(self):
        res = self.__session.get("https://weibo.com/u/%s/home" % (self.__uid)).text
        # print(res)
        if u"我的首页" in res:
            return True
        else:
            return False

    # 进入微博等级页面再回到主页，等级可升到4级
    def check_level(self):
        for i in range(3):
            self.__session.get("http://level.account.weibo.com/level/mylevel?from=front",
                               headers={"Host": "level.account.weibo.com"})
        self.home()

    def __interface_4_all_instructions(self, instruction_name, **kwargs):
        if instruction_name == "comment_forward":
            img_url = kwargs["img_url"] if "img_url" in kwargs else None
            img_id = kwargs["img_id"] if "img_id" in kwargs else None
            forward = kwargs["forward"] if "forward" in kwargs else False
            self.comment_forward(kwargs["blog_mid"], kwargs["text"], forward, img_url, img_id)
        if instruction_name == "post":
            img_url_list = kwargs["img_url_list"] if "img_url_list" in kwargs else None
            img_id_list = kwargs["img_id_list"] if "img_id_list" in kwargs else None
            self.post(kwargs["text"], img_url_list, img_id_list)
        if instruction_name == "like_blog":
            self.like_blog(kwargs["blog_mid"])
        if instruction_name == "like_comment":
            self.like_comment(kwargs["comment_id"])
        if instruction_name == "follow_unfo":
            self.follow_unfo(kwargs["object_uid"], kwargs["follow"])
        if instruction_name == "reply_comment":
            img_url = kwargs["img_url"] if "img_url" in kwargs else None
            img_id = kwargs["img_id"] if "img_id" in kwargs else None
            approval = kwargs["approval"] if "approval" in kwargs else False
            forward = kwargs["forward"] if "forward" in kwargs else False
            self.reply_comment(kwargs["blog_mid"], kwargs["comment_id"], kwargs["text"], approval, forward, img_url, img_id)
        if instruction_name == "del_blog":
            self.del_blog(kwargs["blog_mid"])
        if instruction_name == "del_comment":
            self.del_comment(kwargs["blog_mid"], kwargs["comment_id"])

    def shell(self):
        logging.warning("shell ...")
        while True:
            argv = input()
            if argv == "exit":
                break
            argv = argv.split(" ")
            parameters = ["blog_mid=", "forward=", "img_url=", "img_id=", "text=", "img_url_list=",
                          "img_id_list=", "comment_id=", "object_uid=", "approval="]
            options, args = getopt.getopt(argv, "i:", parameters)
            parameters_inp = ["--{}".format(p.rstrip("=")) for p in parameters]

            instruction_name = ""
            parameters_dict = {}
            for opt, arg in options:
                if opt == "-i":
                    instruction_name = arg
                    print("instruction: {}".format(arg))
                if opt in parameters_inp:
                    if arg == "True":
                        arg = True
                    if arg == "False":
                        arg = False
                    if arg == "None":
                        arg = None

                    if opt == "--img_url_list" or opt == "--img_id_list":
                        arg = arg.split("|")
                    parameters_dict[re.search("--(.*)", opt).group(1)] = arg
            try:
                self.__interface_4_all_instructions(instruction_name, **parameters_dict)
            except Exception as e:
                logging.warning("sth went wrong, {}".format(e))
if __name__ == "__main__":
    weibo_robot = WeiboRobot("15850782585", "Sina6981228.")
    # feedback = weibo_robot.post("Robot Test...", img_url_list=["test1.png", "test2.png"])

    '''
    -i post --text=HellowWorld --img_url_list=test1.png|test2.png
    -i comment_forward --blog_mid=4321252124398928 --text=heyguys --forward=True --img_url=test1.png
    -i like_blog --blog_mid=4321252124398928
    -i like_comment --comment_id=4321252828681425
    -i reply_comment --blog_mid=4321252124398928 --comment_id=4321252828681425 --text=reply_comment --img_url=test1.png
    -i del_comment --blog_mid=4321252124398928 --comment_id=4321252828681425
    -i del_blog --blog_mid=4321252124398928
    '''
    weibo_robot.shell()


import time
import base64
from Doraemon.Others import ext_apis
from Doraemon.Requests import chrome_dora
import os
from tqdm import tqdm
import logging
from pprint import pprint


class OrgInvestigator:
    def __init__(self, api_key):
        self.url_map = {
            "baidu": "https://www.baidu.com/s?&wd={}&ie=utf-8",
            "neris": "http://neris.csrc.gov.cn/shixinchaxun/",
            "mee_gov": "http://www.mee.gov.cn/qwjs2019/?searchword={}",
            "mnr_gov": "http://s.lrn.cn/jsearchfront/search.do?" +
                  "websiteid=110000000000000&pg=1&p=1&searchid=1&tpl=13&cateid=1&q={}" +
                  "&filter=001&x=0&y=0",
            "china_tax": "http://www.chinatax.gov.cn/s?qt={}&siteCode=bm29000002&tab=all&toolsStatus=1",
            "zxgk": "http://zxgk.court.gov.cn/zhzxgk/",
            # "customs": "http://www.customs.gov.cn"
        }
        self.name_map = {
            "baidu": "百度检索结果",
            "neris": "中国证监会证券期货市场失信检索结果",
            "mee_gov": "中华人民共和国生态环境部检索结果",
            "mnr_gov": "中华人民共和国自然资源部检索结果",
            "china_tax": "国家税务局重大税收违法案件信息公布栏检索结果",
            "zxgk": "全国法院被执行人信息检索结果",
            # "customs": "中国海关总署检索结果",
        }

        # for websites need to recognize captcha
        self.ele_map = {
            "neris": {
                "captcha": "img#captcha_img",
                "captcha_inp": "input#ycode",
                "org": "input#objName",
                "submit": "img#querybtn",
                "info": "td.search_bg",
            },
            "zxgk": {
                "captcha": "img#captchaImg",
                "captcha_inp": "input#yzm",
                "org": "input#pName",
                "submit": 'div#yzm-group button[onclick*="search();"]',
                "info": "div#yzm-group",
            }
        }

        # init chrome
        user_ag = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36'
        self.chrome = chrome_dora.MyChrome(headless=True, proxy=None, user_agent=user_ag)
        # filter
        self.api_key = api_key

    def investigate(self,
                    key_word_list,
                    save_dir,
                    switch
                    ):
        switch_ = {key: val for key, val in switch.items() if val is True}
        captcha_error_info = {}
        for key_word in key_word_list:
            for website, val in tqdm(switch_.items(), desc="screenshot {}".format(key_word)):
                if val:
                    url = self.url_map[website]
                    web_name = self.name_map[website]
                    if website not in {"neris", "zxgk", "customs"}:
                        url = url.format(key_word)
                        self.chrome.get(url)
                    # elif website == "customs":
                    #     self.chrome.get(url)
                    #     inp_box = self.chrome.find_element_by_css_selector("input#ess_ctr151088_ListC_Info_ctl00_KEYWORDS")
                    #     inp_box.send_keys(key_word)
                    #     inp_box.send_keys(Keys.ENTER)

                    else:
                        elements = self.ele_map[website]
                        self.chrome.get(url)

                        def refresh():
                            while True:
                                self.chrome.refresh()
                                time.sleep(1)
                                body_text = self.chrome.find_element_by_tag_name("body").text
                                if "找不到您要的页面" not in body_text:
                                    break
                        # screen_shot captcha and recognize it
                        # self.chrome.wait_to_get_element(elements["captcha"], 5)

                        def input_kw_n_captch():
                            # input organization name
                            refresh()
                            self.chrome.find_element_by_css_selector(elements["org"]).send_keys(key_word)
                            time.sleep(0.5)
                            captcha_inp = self.chrome.find_element_by_css_selector(elements["captcha_inp"])
                            captcha_inp.click()

                            self.chrome.find_element_by_css_selector(elements["captcha"]).screenshot('{}/captcha.png'.format(save_dir))
                            captcha_bin = open("{}/captcha.png".format(save_dir), "rb").read()
                            captcha_bs64 = base64.b64encode(captcha_bin).decode()
                            captcha_code = ext_apis.captcha_rec_3023data(captcha_bs64,
                                                                         format="png",
                                                                         type=5001,
                                                                         api_key=self.api_key)
                            # input captcha code
                            captcha_inp.send_keys(captcha_code)
                            # click to submit
                            self.chrome.find_element_by_css_selector(elements["submit"]).click()
                            time.sleep(1)

                        input_kw_n_captch()
                        while True:
                            body_text = self.chrome.find_element_by_tag_name("body").text
                            if "验证码错误" in body_text:
                                logging.warning("验证码错误，正在重试...".format())
                                # log error num
                                if key_word not in captcha_error_info:
                                    captcha_error_info[key_word] = {}
                                captcha_error_info[key_word][web_name] = captcha_error_info[key_word].get(web_name, 0) + 1
                                input_kw_n_captch()
                            else:
                                break

                    save_path = os.path.join(save_dir, web_name)
                    if not os.path.exists(save_path):
                        os.makedirs(save_path)
                    save_path = os.path.join(save_path, "{}.jpg".format(key_word))
                    self.chrome.screenshot_current_page(save_path)

        if len(captcha_error_info) > 0:
            print("本轮检索累积验证码错误次数统计如下，注意检查账户余额。")
            pprint(captcha_error_info)

    def quite(self):
        # quit
        self.chrome.quit()


if __name__ == "__main__":
    key_word_list = ["山东华骜植化", "华泰证券", "广发证券", "海通证券"]
    api_key = "7dab451889b5f88db606750d615d9824"
    save_dir = "./temp"
    switch = {
        "baidu": True,
        "neris": True,
        "mee_gov": True,
        "mnr_gov": True,
        "china_tax": True,
        "zxgk": True,
        # "customs": True,
    }
    investigator = OrgInvestigator(api_key)
    investigator.investigate(key_word_list, save_dir, switch)



'''
the code is invalid @ 2020.10.09
'''
import random
import time
from bs4 import BeautifulSoup
from urllib import parse
import re
from Doraemon.Requests import requests_dora, proxies_dora
from tqdm import tqdm
import logging

# ORG_KEYWORDS = ["college", "company", "university", "school", "corporation",
#                     "institute", "organization", "association"]

def quote(queryStr):
    try:
        queryStr = parse.quote(queryStr)
    except:
        queryStr = parse.quote(queryStr.encode('utf-8', 'ignore'))

    return queryStr


def google_search(queryStr, get_proxies_fun = None, page = 1):
    url = 'https://www.google.com/search?biw=1920&safe=active&hl=en&q=%s&oq=%s&start=%d' % (queryStr, queryStr, (page-1) * 10)

    response = requests_dora.try_best_2_get(url, headers=requests_dora.get_default_headers(), invoked_by="google_search", get_proxies_fun=get_proxies_fun, timeout=60)
    status = response.status_code
    if status == 200:
        html = response.text
    else:
        print("status: {}, try again....".format(status))
        random.seed(time.time())
        time.sleep(3 + 5 * random.random())
        return google_search(queryStr, page, get_proxies_fun)

    return html


def get_entity(query_str, get_proxies_fun, wait=1.5):

    rel_org_name_set = set()
    logging.warning("start crawling {}...".format(query_str))

    text = google_search(query_str, get_proxies_fun)
    random.seed(time.time())
    time.sleep(wait * random.random())

    soup = BeautifulSoup(text, "lxml")

    # is there an entity in google KG?
    div_kg_hearer = soup.select_one("div.kp-header")

    if div_kg_hearer is None: # if there is no knowledge graph at the right, drop it
        logging.warning("no entity returned for this query")
        return None

    enti_name = div_kg_hearer.select_one("div[role=heading] span")
    enti_name = enti_name.text if enti_name is not None else None
    if enti_name is None or "..." in enti_name:
        se = re.search('\["t-dhmk9MkDbvI",.*\[\["data",null,null,null,null,\[null,"\[\\\\"(.*)\\\\",', text)
        if se is not None:
            enti_name = se.group(1)
        else:
            logging.warning("sth went wrong when extracting the name of the entity")
            return None

    # identify the type
    span_list = div_kg_hearer.select("span")
    enti_type = span_list[-1].text if len(span_list) > 1 else "unknown"

    # description from wikipedia
    des = soup.find("h3", text="Description")
    des_info = ""
    if des is not None:
        des_span = des.parent.select_one("span")
        des_info = des_span.text if des_span is not None else ""

    # extract attributes
    attr_tags = soup.select("div.Z1hOCe")
    attr_dict = {}
    for attr in attr_tags:
        attr_str = attr.get_text()
        se = re.search("(.*?)[:：](.*)", attr_str)
        if se is None:
            continue
        key_attr = se.group(1)
        val_attr = se.group(2)
        attr_dict[key_attr] = val_attr

    # relevant org name on current page
    a_reltype_list = soup.select("div.MRfBrb > a")
    for a in a_reltype_list:
        rel_org_name_set.add(a["title"].strip())

    # collect next urls e.g. : more x+
    div_list = soup.select("div.yp1CPe")
    next = []
    host = "https://www.google.com"
    for div in div_list:
        a_list = div.select("a.EbH0bb")
        for a in a_list:
            if "http" not in a["href"]:
                next.append("%s%s" % (host, a["href"]))

    # crawl parent org
    a_parent_org = soup.find("a", text="Parent organization")
    if a_parent_org is not None:
        parent_str = a_parent_org.parent.parent.text.strip()
        parent_org = parent_str.split(":")[1]
        rel_org_name_set.add(parent_org.strip())

    # crawl subsidiaries
    a_subsidiaries = soup.find("a", text="Subsidiaries")
    if a_subsidiaries is not None:
        href = a_subsidiaries["href"]
        if "http" not in href:
            subsidiaries_str = a_subsidiaries.parent.parent.text.strip()
            subs = subsidiaries_str.split(":")[1].split(",")
            for sub in subs:
                sub = sub.strip()
                if sub == "MORE":
                    continue
                rel_org_name_set.add(sub)
            next.append("%s%s" % (host, href))

    # scrawl urls in list 'next'
    for url in tqdm(next, desc="crawling relevant org names..."):
        res = requests_dora.try_best_2_get(url, invoked_by="get_org_name", headers=requests_dora.get_default_headers(), get_proxies_fun=get_proxies_fun)
        soup = BeautifulSoup(res.text, "lxml")

        # crawl items at the top
        a_list = soup.select("a.klitem")
        for a in a_list:
            rel_org_name = a["title"]
            rel_org_name_set.add(rel_org_name.strip())

        # crawl headings under the map if any
        heading_list = soup.select("div.VkpGBb")
        for heading in heading_list:
            heading_str = heading.select_one("div[role='heading']")
            rel_org_name_set.add(heading_str.get_text())

        random.seed(time.time())
        bar.update()
        time.sleep(wait * random.random())

    rel_org_name_list = [org_name for org_name in rel_org_name_set if len(org_name) > 1]
    return {"query_str": query_str, "name": enti_name, "type": enti_type,
            "des": des_info, "attributes": attr_dict, "rel_org": rel_org_name_list}

if __name__ == "__main__":
    def get_proxy():
        proxy_str = "127.0.0.1:1080"
        proxies = {"http": "http://%s" % proxy_str,
                   "https": "http://%s" % proxy_str, }
        return proxies
    query_str = "阿里巴巴"
    # html = google_search(query_str, get_proxies_fun = get_proxy)
    # print(html)
    entity = get_entity(query_str, get_proxies_fun = get_proxy)
    print(entity)


'''
character decoding is invalid now
'''
from Doraemon import requests_dora
from bs4 import BeautifulSoup
import re
import time
import logging
import random
import json


def get_around(city_id, center_shop_id, dis, page, get_proxies_fun=None):
    '''
    get shops in the vicinity of the center shop
    :param city_id: the id of the target city, e.g. 2 -> Beijing
    :param center_shop_id: shop id of the center shop
    :param dis: the max distance of shops from the center shop
    :param page: page index [1, 50]
    :return: a list of shops
    e.g.
    [
      {
        "img_src": "https://img.meituan.net/msmerchant/2e5787325ba4579ec2e2e3f45038ade1149446.jpg%40340w_255h_1e_1c_1l%7Cwatermark%3D1%26%26r%3D1%26p%3D9%26x%3D2%26y%3D2%26relative%3D1%26o%3D20",
        "title": "速度披萨(华贸城店)",
        "star_level": 4.5,
        "review_num": 30,
        "mean_price": 89,
        "cat": "西餐",
        "region": "北苑家园",
        "addr": "清苑路13号",
        "rec_dish": [
          "黑芝麻沙拉",
          "蟹肉意面",
          "火腿榴莲披萨双拼"
        ],
        "score": {
          "taste": 8.5,
          "env": 8.4,
          "service": 8.4
        }
      },
    ]
    '''

    url = "http://www.dianping.com/search/around/{}/10_{}/d{}p{}".format(city_id, center_shop_id, dis, page)
    print(url)
    headers = requests_dora.get_default_headers()
    headers["Accept"] = 'application/json, text/javascript'
    headers["User-Agent"] = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36"
    headers["Host"] = "www.dianping.com"
    del headers["Referer"]

    while True:
        res = requests_dora.try_best_2_get(url, get_proxies_fun=get_proxies_fun, headers=headers)
        if res.status_code == 200:
            break
        random.seed(time.time())
        sleep_time = 3 + 7 * random.random()
        logging.warning("failde ... try again in {} s...".format(sleep_time))
        time.sleep(sleep_time)

    # decrypt: invalid @ 2020.10.09
    html = res.text
    if "没有找到" in html:
        logging.warning("can not find any shops...")
        return None

    url = "https:{}".format(re.search('href="(//s3plus\.meituan\.net.*?\.css)">', html).group(1))
    map_code2char = get_map_code2char(url)
    it = re.finditer('<span class="([0-9a-zA-Z]*?)"></span>', html)
    for match in it:
        code = match.group(1)
        if code in map_code2char:
            html = re.sub('<span class="{}"></span>'.format(code), map_code2char[code], html)

    soup = BeautifulSoup(html, "lxml")
    li_list = soup.select("div.shop-list > ul > li")

    # crawl
    shop_list = []
    for li in li_list:
        shop = {}
        try:
            img = li.select_one("div.pic img")
            img_src = img["src"]
            shop["img_src"] = img_src
        except:
            pass

        try:
            txt = li.select_one("div.txt")
            title = txt.select_one("div.tit > a")["title"]
            shop["title"] = title
        except:
            continue

        try:
            comment = txt.select_one("div.comment")
            star_span = str(comment.select_one("span.sml-rank-stars"))
            star_level = float(re.search("sml-str(\d+)", star_span).group(1)) / 10
            shop["star_level"] = star_level
            review_num = int(comment.select_one("a.review-num > b").get_text())
            shop["review_num"] = review_num
            mean_price = comment.select_one("a.mean-price > b").get_text()
            mean_price = int(re.search("(\d+)", mean_price).group(1))
            shop["mean_price"] = mean_price
        except:
            pass

        try:
            tag_addr = txt.select_one("div.tag-addr")
            category = tag_addr.select_one('a[data-click-name="shop_tag_cate_click"] > span.tag').get_text()
            region = tag_addr.select_one('a[data-click-name="shop_tag_region_click"] > span.tag').get_text()
            addr = tag_addr.select_one("span.addr").get_text()
            shop["cat"] = category
            shop["region"] = region
            shop["addr"] = addr
        except:
            pass

        try:
            rec_dish = txt.select("div.recommend > a")
            assert len(rec_dish) > 0
            rec_dish = [dish.get_text() for dish in rec_dish]
            shop["rec_dish"] = rec_dish
        except:
            pass

        try:
            comment_list = txt.select("span.comment-list > span > b")
            score_list = [float(sc.get_text()) for sc in comment_list]
            shop["score"] = {
                "taste": score_list[0],
                "env": score_list[1],
                "service": score_list[2],
            }
        except:
            pass

        shop_list.append(shop)
    return shop_list


def search_shops(city_id, keyword, page, get_proxies_fun=None):
    '''
    seach shops by keyword
    :param city_id: the id of the target city
    :param keyword: the keyword
    :param page: page index [1, 50]
    :return: a list of shops : [{"name": "shopname1", "shop_id": "1245587}, ...]
    '''
    url = "http://www.dianping.com/search/keyword/{}/0_{}/p{}".format(city_id, keyword, page)
    headers = requests_dora.get_default_headers()
    headers["Accept"] = 'application/json, text/javascript'
    headers["User-Agent"] = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36"
    headers["Host"] = "www.dianping.com"
    del headers["Referer"]

    while True:
        res = requests_dora.try_best_2_get(url, get_proxies_fun=get_proxies_fun, headers=headers)
        if res.status_code == 200:
            break
        random.seed(time.time())
        sleep_time = 5 + 10 * random.random()
        logging.warning("failde ... try again in {} s...".format(sleep_time))
        time.sleep(sleep_time)

    html = res.text
    if "没有找到" in html:
        logging.warning("can not find any shops...")
        return None

    soup = BeautifulSoup(html, "lxml")
    li_list = soup.select("div.shop-list > ul > li")

    shop_list = []
    for li in li_list:
        shop = {}
        try:
            txt = li.select_one("div.txt")
            a_title = txt.select_one("div.tit > a")
            title = a_title["title"]
            shop_id = a_title["data-shopid"]
            shop["name"] = title
            shop["shop_id"] = shop_id
        except:
            continue
        shop_list.append(shop)

    return shop_list


def get_map_code2char(url):
    '''
    get the map for decrypting
    :param url: the url of the css file which is used for encrypt
    :return:
    '''
    headers = requests_dora.get_default_headers()
    headers["Accept"] = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8"
    res = requests_dora.try_best_2_get(url, headers=headers)
    css = res.text
    it = re.finditer('span\[class\^="(.*?)"\]{.*?background-image:.*?url\((.*?)\)', css)

    map_key2char_global = {}
    for match in it:
        # extract the css selector
        css_sel_head = match.group(1)

        # extract the text
        svg_url = "https:{}".format(match.group(2))
        svg = requests_dora.try_best_2_get(svg_url, headers=headers)
        soup = BeautifulSoup(svg.text, "lxml")
        text_list = soup.select("text")
        text_list = [txt.text for txt in text_list]
        text = "".join(text_list)
        text = re.sub("\n", "", text)

        # map selectors to character
        pattern = "\.(%s.*?){background:(.*?)px (.*?)px;}" % css_sel_head
        it = re.finditer(pattern, css)
        map_char2pos = {}
        for match in it:
            key = match.group(1)
            y = match.group(2)
            x = match.group(3)
            map_char2pos[key] = (float(x), float(y))

        key_list = list(sorted(map_char2pos.keys(), key=lambda k: map_char2pos[k], reverse=True))
        tpl = zip(key_list, text)
        map_key2char = dict((i, j) for i, j in tpl)
        map_key2char_global = {**map_key2char_global, **map_key2char}
    return map_key2char_global


if __name__ == "__main__":
    # shop_list = search_shops("2", "4s店", 1)
    # print(shop_list)

    # get shop list around the center shop
    shop_list = get_around("2", "5724615", 2000, 1)
    print(json.dumps(shop_list, indent=2, ensure_ascii=False))

    # # get the map to decrypt
    # url = "https://s3plus.meituan.net/v1/mss_0a06a471f9514fc79c981b5466f56b91/svgtextcss/8d42683a9b290707dcf319a5920cff72.css"
    # map1 = get_map_code2char(url)
    # print(map1)
    pass



# Requests
from Doraemon.Requests import chrome_dora
from Doraemon.Requests import requests_dora
from Doraemon.Requests import proxies_dora

# Crawlers
from Doraemon.Crawlers import google_KG
from Doraemon.Crawlers import google_translator
from Doraemon.Crawlers import dianping
from Doraemon.Crawlers import whois
from Doraemon.Crawlers import qq_music_crawler_by_album, qq_music_crawler_by_area
from Doraemon.Crawlers import investigation

# Other
from Doraemon.Others import domain2ip, ext_apis



import logging

class Logger:
    def __init__(self, path, clevel=logging.DEBUG, Flevel=logging.DEBUG):
        self.path = path

        self.logger = logging.getLogger(path)
        self.logger.setLevel(logging.DEBUG)
        fmt = logging.Formatter('[%(asctime)s] [%(levelname)s] %(message)s', '%Y-%m-%d %H:%M:%S')
        # 设置CMD日志
        sh = logging.StreamHandler()
        sh.setFormatter(fmt)
        sh.setLevel(clevel)  # 设置cmd记录的起始等级
        # 设置文件日志
        fh = logging.FileHandler(path, encoding="UTF-8")
        fh.setFormatter(fmt)
        fh.setLevel(Flevel)  # 设置文件记录的起始等级
        self.logger.addHandler(sh)
        self.logger.addHandler(fh)

    def debug(self, message):
        self.logger.debug("->%s; debug: %s" % (self.path, message))

    def info(self, message):
        self.logger.info("->%s; info: %s" % (self.path, message))

    def warning(self, message):
        self.logger.warning("->%s; warning: %s" % (self.path, message))

    def error(self, message):
        self.logger.error("->%s; error: %s" % (self.path, message))

    def critical(self, message):
        self.logger.critical("->%s; critical: %s" % (self.path, message))


if __name__ == '__main__':
    logyyx = Logger('sofa.log', logging.DEBUG, logging.DEBUG)
    logyyx.debug(u'一个debug信息')
    logyyx.info(u'一个info信息')
    logyyx.warning(u'一个warning信息')
    logyyx.error(u'一个error信息')
    logyyx.critical(u'一个致命critical信息')


import re
import sys
import socket
import queue
import threading


class Parser(threading.Thread):
    def __init__(self, queue, domain_name2ip, max_fail_num=0):
        threading.Thread.__init__(self)
        self._queue = queue
        self.domain_name2ip = domain_name2ip
        self.domain_name2fail_num = {}
        self.max_fail_num = max_fail_num

    def run(self):
        while not self._queue.empty():
            domain_name = self._queue.get()
            try:
                ip = socket.gethostbyname(domain_name)
                self.domain_name2ip[domain_name] = ip
                sys.stdout.write("\r{} in queue, {} -> {} ---------------------------".format(self._queue.qsize(), domain_name, ip))
                sys.stdout.flush()

            except Exception as e:
                fail_num = self.domain_name2fail_num.get(domain_name, 0)
                if fail_num < self.max_fail_num:
                    self._queue.put(domain_name)
                else:
                    self.domain_name2ip[domain_name] = None

                self.domain_name2fail_num[domain_name] = fail_num + 1
                sys.stdout.write(
                    "\r{} in queue, {} failed: {}/{} ----------------------".format(self._queue.qsize(), domain_name, fail_num, self.max_fail_num))
                sys.stdout.flush()


def urlfillter(url):
    return re.sub("https?://", "", url).split("/")[0].split(":")[0]


def gethostbyname_fast(url_list, domain_name2ip, thread_num = 20, max_fail_num = 0):
    domain_names = {urlfillter(url.strip()) for url in url_list}

    uni_queue = queue.Queue()
    for dn in domain_names:
        uni_queue.put(dn)

    workers = []
    for _ in range(thread_num):
        worker = Parser(uni_queue, domain_name2ip, max_fail_num)
        worker.start()
        workers.append(worker)

    for worker in workers:
        worker.join()


if __name__ == '__main__':
    threads = 100
    max_fail_num = 0
    domain_name2ip = {}
    url_list = ["https://www.baidu.com", "https://www.qq.com"]
    gethostbyname_fast(url_list, domain_name2ip, threads, max_fail_num)



import base64
from urllib import parse
import requests
import json
from Doraemon import requests_dora, proxies_dora
import logging


def img_orc_google(img_url, google_api_key):
    '''
    :param img_url: http style or local path
    :return: text
    '''
    api = "https://vision.googleapis.com/v1/images:annotate?key=%s" % google_api_key
    data = {
        "requests": [
            {
                "image": {
                    "source": {
                        "imageUri": img_url,
                    }
                },
                "features": [
                    {
                        "type": "DOCUMENT_TEXT_DETECTION"
                    }
                ]
            }
        ]
    }
    if "http" not in img_url: # local path
        img = open(img_url, "rb")
        img_bs64 = base64.b64encode(img.read()).decode()
        data["requests"][0]["image"] = {"content": img_bs64}

    res = requests_dora.try_best_2_post(api, headers=requests_dora.get_default_headers(),
                                        proxies=proxies_dora.get_proxies("127.0.0.1:1080"),
                                        data=json.dumps(data), invoked_by="img_orc_google")
    if res.status_code == 200:
        json_res = json.loads(res.text)["responses"][0]
        if "error" in json_res:
            logging.warning("%s: %s" % (img_url, json_res["error"]["message"]))
            return None
        else:
            try:
                text = json_res["textAnnotations"][0]["description"]
                logging.info("google ocr success!")
                return text
            except Exception:
                return None
    logging.warning("bad response, status_code: %d" % res.status_code)
    return None


def img_orc_baidu(filePath, baidu_api, accurate=False):
    '''
    Baidu OCR API
    BAIDU_API_OCR_ACCURATE 500/d
    BAIDU_API_OCR_GENERAL 50000/d  it is bad
    :param filePath: local path
    :return: text in the image
    '''
    img = open(filePath, "rb")
    url = "https://aip.baidubce.com/rest/2.0/ocr/v1/general_basic?access_token={}".format(baidu_api)
    if accurate:
        url = "https://aip.baidubce.com/rest/2.0/ocr/v1/accurate_basic?access_token={}".format(baidu_api)
    img_bs64 = base64.b64encode(img.read()).decode()
    data = {
        "language_type": "ENG",
        "detect_direction": "true",
        "probability": "true",
        "image": img_bs64
    }
    data = parse.urlencode(data)
    res = requests.post(url, data=data)
    # print(res.text)
    return json.loads(res.text)


if __name__ == "__main__":
    text = img_orc_google("./1.tif", "AIzaSyBuvFKna_9YqhszzmGNV1MIFjGNnfz8uyk")
    print(text)


import requests
import json


def captcha_rec_3023data(captcha_bs64, format, type, api_key):
    captcha_api = "http://api.3023data.com/ocr/captcha"
    res = requests.post(captcha_api, data={
        "type": type,
        "image": "data:image/{};base64,{}".format(format, captcha_bs64),
    }, headers={
        "key": api_key,
    })
    captcha_res = json.loads(res.text)
    status = captcha_res["code"]
    if status != 0:
        print(captcha_res)
        raise Exception
    else:
        captcha_code = captcha_res["data"]["captcha"]
    return captcha_code